# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CryptoCreateOfframpParams"]


class CryptoCreateOfframpParams(TypedDict, total=False):
    currency: Required[Literal["usdt", "usdc"]]

    payment_rail: Required[Annotated[Literal["ach", "wire"], PropertyInfo(alias="paymentRail")]]

    virtual_account_id: Required[Annotated[str, PropertyInfo(alias="virtualAccountId")]]
    """The id of the virtual account to offramp funds to."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from .money import Money
from .._models import BaseModel

__all__ = ["CommissionDetails", "UnionMember0", "UnionMember1"]


class UnionMember0(BaseModel):
    amount: Money
    """
    The amount of money to be diverted from the virtual account if the type of rule
    is flat_fee.
    """

    frequency: Literal["monthly", "yearly"]
    """The frequency from the start date that the rule will be applied.

    E.g. a start date of November 26, 2024 with a frequency of "monthly" will have
    the rule applied on December 26, 2024. If the frequency was "yearly", the rule
    would be applied again on November 26, 2025.
    """

    type: Literal["flatFee"]
    """The type of the commission rule.

    A type of flat_fee will take a flat fee from the virtual account and transfer it
    to the primary account with the frequency of the collection defined as monthly
    or yearly.
    """

    start_date: Optional[datetime] = FieldInfo(alias="startDate", default=None)
    """The date that the rule will begin to take effect."""


class UnionMember1(BaseModel):
    take_rate: float = FieldInfo(alias="takeRate")
    """
    The percentage of funds to divert from the virtual account to the primary
    account.
    """

    type: Literal["takeRate"]
    """The type of the commission rule.

    A type of take_rate will take a percent of the virtual account's funds and
    divert them to the primary account.
    """


CommissionDetails: TypeAlias = Union[UnionMember0, UnionMember1]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import DeveloperApplicationType, developer_account_create_application_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.developer_application_type import DeveloperApplicationType
from ..types.developer_application_model import DeveloperApplicationModel
from ..types.developer_application_data_param import DeveloperApplicationDataParam

__all__ = ["DeveloperAccountResource", "AsyncDeveloperAccountResource"]


class DeveloperAccountResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DeveloperAccountResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return DeveloperAccountResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DeveloperAccountResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return DeveloperAccountResourceWithStreamingResponse(self)

    def create_application(
        self,
        developer_account_id: str,
        *,
        data: DeveloperApplicationDataParam,
        name: str,
        type: DeveloperApplicationType,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DeveloperApplicationModel:
        """
        Create a DeveloperApplication

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not developer_account_id:
            raise ValueError(
                f"Expected a non-empty value for `developer_account_id` but received {developer_account_id!r}"
            )
        return self._post(
            f"/developer-account/{developer_account_id}/application",
            body=maybe_transform(
                {
                    "data": data,
                    "name": name,
                    "type": type,
                },
                developer_account_create_application_params.DeveloperAccountCreateApplicationParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeveloperApplicationModel,
        )


class AsyncDeveloperAccountResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDeveloperAccountResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDeveloperAccountResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDeveloperAccountResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncDeveloperAccountResourceWithStreamingResponse(self)

    async def create_application(
        self,
        developer_account_id: str,
        *,
        data: DeveloperApplicationDataParam,
        name: str,
        type: DeveloperApplicationType,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DeveloperApplicationModel:
        """
        Create a DeveloperApplication

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not developer_account_id:
            raise ValueError(
                f"Expected a non-empty value for `developer_account_id` but received {developer_account_id!r}"
            )
        return await self._post(
            f"/developer-account/{developer_account_id}/application",
            body=await async_maybe_transform(
                {
                    "data": data,
                    "name": name,
                    "type": type,
                },
                developer_account_create_application_params.DeveloperAccountCreateApplicationParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DeveloperApplicationModel,
        )


class DeveloperAccountResourceWithRawResponse:
    def __init__(self, developer_account: DeveloperAccountResource) -> None:
        self._developer_account = developer_account

        self.create_application = to_raw_response_wrapper(
            developer_account.create_application,
        )


class AsyncDeveloperAccountResourceWithRawResponse:
    def __init__(self, developer_account: AsyncDeveloperAccountResource) -> None:
        self._developer_account = developer_account

        self.create_application = async_to_raw_response_wrapper(
            developer_account.create_application,
        )


class DeveloperAccountResourceWithStreamingResponse:
    def __init__(self, developer_account: DeveloperAccountResource) -> None:
        self._developer_account = developer_account

        self.create_application = to_streamed_response_wrapper(
            developer_account.create_application,
        )


class AsyncDeveloperAccountResourceWithStreamingResponse:
    def __init__(self, developer_account: AsyncDeveloperAccountResource) -> None:
        self._developer_account = developer_account

        self.create_application = async_to_streamed_response_wrapper(
            developer_account.create_application,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CardGroupListParams"]


class CardGroupListParams(TypedDict, total=False):
    cursor: str
    """A cursor string to fetch the next page of results"""

    filter_name: Annotated[str, PropertyInfo(alias="filter:name")]
    """Pass in a name to filter for card groups with a matching name."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..money import Money
from ..._models import BaseModel
from .restriction import Restriction

__all__ = [
    "SpendingConstraint",
    "CountryRule",
    "MerchantCategoryCodeRule",
    "MerchantCategoryRule",
    "MerchantRule",
    "SpendingRule",
    "SpendingRuleTransactionSizeLimit",
    "SpendingRuleUtilizationLimit",
    "SpendingRuleUtilizationLimitV2",
]


class CountryRule(BaseModel):
    countries: List[str]
    """A 2 digit country code"""

    restriction: Restriction


class MerchantCategoryCodeRule(BaseModel):
    merchant_category_codes: List[str] = FieldInfo(alias="merchantCategoryCodes")

    restriction: Restriction


class MerchantCategoryRule(BaseModel):
    merchant_categories: List[str] = FieldInfo(alias="merchantCategories")

    restriction: Restriction


class MerchantRule(BaseModel):
    merchants: List[str]

    restriction: Restriction


class SpendingRuleTransactionSizeLimit(BaseModel):
    maximum: Optional[Money] = None
    """Represents a monetary value"""

    minimum: Optional[Money] = None
    """Represents a monetary value"""


class SpendingRuleUtilizationLimit(BaseModel):
    limit_amount: Money = FieldInfo(alias="limitAmount")
    """Represents a monetary value"""

    preset: Literal["daily", "weekly", "monthly", "yearly", "collective"]

    start_date: Optional[str] = FieldInfo(alias="startDate", default=None)
    """Format ISO-8601.

    A day that equals today or the past. This is optional.If the `preset` is
    "daily", this value is ignored. If the `preset` is "weekly", "monthly" or
    "yearly", then the this value is used to compute when the limit should start
    limit.
    """

    timezone: Optional[str] = None
    """IANA timezone string.

    Limits always reset at midnight of the timezone specified. If no timezone is
    specified, then UTC time is used.
    """


class SpendingRuleUtilizationLimitV2(BaseModel):
    limit_amount: Money = FieldInfo(alias="limitAmount")
    """Represents a monetary value"""

    preset: Literal["daily", "weekly", "monthly", "yearly", "collective"]

    start_date: Optional[str] = FieldInfo(alias="startDate", default=None)
    """Format ISO-8601.

    A day that equals today or the past. This is optional.If the `preset` is
    "daily", this value is ignored. If the `preset` is "weekly", "monthly" or
    "yearly", then the this value is used to compute when the limit should start
    limit.
    """

    timezone: Optional[str] = None
    """IANA timezone string.

    Limits always reset at midnight of the timezone specified. If no timezone is
    specified, then UTC time is used.
    """


class SpendingRule(BaseModel):
    transaction_size_limit: Optional[SpendingRuleTransactionSizeLimit] = FieldInfo(
        alias="transactionSizeLimit", default=None
    )

    utilization_limit: Optional[SpendingRuleUtilizationLimit] = FieldInfo(alias="utilizationLimit", default=None)

    utilization_limit_v2: Optional[List[SpendingRuleUtilizationLimitV2]] = FieldInfo(
        alias="utilizationLimitV2", default=None
    )


class SpendingConstraint(BaseModel):
    country_rule: Optional[CountryRule] = FieldInfo(alias="countryRule", default=None)

    merchant_category_code_rule: Optional[MerchantCategoryCodeRule] = FieldInfo(
        alias="merchantCategoryCodeRule", default=None
    )

    merchant_category_rule: Optional[MerchantCategoryRule] = FieldInfo(alias="merchantCategoryRule", default=None)

    merchant_rule: Optional[MerchantRule] = FieldInfo(alias="merchantRule", default=None)

    spending_rule: Optional[SpendingRule] = FieldInfo(alias="spendingRule", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .money import Money
from .._models import BaseModel

__all__ = ["CardGroupUtilization"]


class CardGroupUtilization(BaseModel):
    spend: Money
    """The amount of money spent in the current period"""

    available_balance: Optional[Money] = FieldInfo(alias="availableBalance", default=None)
    """The amount of money available in the current period.

    Only returned if the card or card group has a spend limit.
    """

    next_reset_date: Optional[datetime] = FieldInfo(alias="nextResetDate", default=None)
    """The date the next reset will occur, undefined if collective"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .money import Money
from .._models import BaseModel
from .commission_rule import CommissionRule
from .virtual_account_model import VirtualAccountModel

__all__ = ["VirtualAccount"]


class VirtualAccount(BaseModel):
    balance: Money
    """The total balance of the virtual account"""

    spend: Money
    """The total spend of the virtual account"""

    virtual_account: VirtualAccountModel = FieldInfo(alias="virtualAccount")
    """The virtual account itself"""

    commission_rule: Optional[CommissionRule] = FieldInfo(alias="commissionRule", default=None)
    """The commission rule applied to the virtual account, if any"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from slash_sdk import SlashSDK, AsyncSlashSDK
from tests.utils import assert_matches_type
from slash_sdk.types import SlashHandle, PaySendResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPay:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: SlashSDK) -> None:
        pay = client.pay.retrieve()
        assert_matches_type(SlashHandle, pay, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: SlashSDK) -> None:
        response = client.pay.with_raw_response.retrieve()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pay = response.parse()
        assert_matches_type(SlashHandle, pay, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: SlashSDK) -> None:
        with client.pay.with_streaming_response.retrieve() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pay = response.parse()
            assert_matches_type(SlashHandle, pay, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_send(self, client: SlashSDK) -> None:
        pay = client.pay.send(
            amount_cents=0,
            slash_handle="slashHandle",
        )
        assert_matches_type(PaySendResponse, pay, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_send_with_all_params(self, client: SlashSDK) -> None:
        pay = client.pay.send(
            amount_cents=0,
            slash_handle="slashHandle",
            legal_entity_id="legalEntityId",
            source_slash_handle_id="sourceSlashHandleId",
        )
        assert_matches_type(PaySendResponse, pay, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_send(self, client: SlashSDK) -> None:
        response = client.pay.with_raw_response.send(
            amount_cents=0,
            slash_handle="slashHandle",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pay = response.parse()
        assert_matches_type(PaySendResponse, pay, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_send(self, client: SlashSDK) -> None:
        with client.pay.with_streaming_response.send(
            amount_cents=0,
            slash_handle="slashHandle",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pay = response.parse()
            assert_matches_type(PaySendResponse, pay, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncPay:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSlashSDK) -> None:
        pay = await async_client.pay.retrieve()
        assert_matches_type(SlashHandle, pay, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.pay.with_raw_response.retrieve()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pay = await response.parse()
        assert_matches_type(SlashHandle, pay, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.pay.with_streaming_response.retrieve() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pay = await response.parse()
            assert_matches_type(SlashHandle, pay, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_send(self, async_client: AsyncSlashSDK) -> None:
        pay = await async_client.pay.send(
            amount_cents=0,
            slash_handle="slashHandle",
        )
        assert_matches_type(PaySendResponse, pay, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_send_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        pay = await async_client.pay.send(
            amount_cents=0,
            slash_handle="slashHandle",
            legal_entity_id="legalEntityId",
            source_slash_handle_id="sourceSlashHandleId",
        )
        assert_matches_type(PaySendResponse, pay, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_send(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.pay.with_raw_response.send(
            amount_cents=0,
            slash_handle="slashHandle",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pay = await response.parse()
        assert_matches_type(PaySendResponse, pay, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_send(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.pay.with_streaming_response.send(
            amount_cents=0,
            slash_handle="slashHandle",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pay = await response.parse()
            assert_matches_type(PaySendResponse, pay, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from slash_sdk import SlashSDK, AsyncSlashSDK
from tests.utils import assert_matches_type
from slash_sdk.types import (
    VirtualAccount,
    VirtualAccountListResponse,
    VirtualAccountCreateResponse,
    VirtualAccountUpdateResponse,
)
from slash_sdk._utils import parse_datetime

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestVirtualAccount:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: SlashSDK) -> None:
        virtual_account = client.virtual_account.create(
            account_id="accountId",
            name="name",
        )
        assert_matches_type(VirtualAccountCreateResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: SlashSDK) -> None:
        virtual_account = client.virtual_account.create(
            account_id="accountId",
            name="name",
            commission_details={
                "amount": {"amount_cents": 0},
                "frequency": "monthly",
                "type": "flatFee",
                "start_date": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
        )
        assert_matches_type(VirtualAccountCreateResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: SlashSDK) -> None:
        response = client.virtual_account.with_raw_response.create(
            account_id="accountId",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = response.parse()
        assert_matches_type(VirtualAccountCreateResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: SlashSDK) -> None:
        with client.virtual_account.with_streaming_response.create(
            account_id="accountId",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = response.parse()
            assert_matches_type(VirtualAccountCreateResponse, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: SlashSDK) -> None:
        virtual_account = client.virtual_account.retrieve(
            "virtualAccountId",
        )
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: SlashSDK) -> None:
        response = client.virtual_account.with_raw_response.retrieve(
            "virtualAccountId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = response.parse()
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: SlashSDK) -> None:
        with client.virtual_account.with_streaming_response.retrieve(
            "virtualAccountId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = response.parse()
            assert_matches_type(VirtualAccount, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `virtual_account_id` but received ''"):
            client.virtual_account.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update_overload_1(self, client: SlashSDK) -> None:
        virtual_account = client.virtual_account.update(
            virtual_account_id="virtualAccountId",
            action="update",
        )
        assert_matches_type(VirtualAccountUpdateResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params_overload_1(self, client: SlashSDK) -> None:
        virtual_account = client.virtual_account.update(
            virtual_account_id="virtualAccountId",
            action="update",
            commission_details={
                "amount": {"amount_cents": 0},
                "frequency": "monthly",
                "type": "flatFee",
                "start_date": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            name="name",
        )
        assert_matches_type(VirtualAccountUpdateResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update_overload_1(self, client: SlashSDK) -> None:
        response = client.virtual_account.with_raw_response.update(
            virtual_account_id="virtualAccountId",
            action="update",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = response.parse()
        assert_matches_type(VirtualAccountUpdateResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update_overload_1(self, client: SlashSDK) -> None:
        with client.virtual_account.with_streaming_response.update(
            virtual_account_id="virtualAccountId",
            action="update",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = response.parse()
            assert_matches_type(VirtualAccountUpdateResponse, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update_overload_1(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `virtual_account_id` but received ''"):
            client.virtual_account.with_raw_response.update(
                virtual_account_id="",
                action="update",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update_overload_2(self, client: SlashSDK) -> None:
        virtual_account = client.virtual_account.update(
            virtual_account_id="virtualAccountId",
            action="close",
        )
        assert_matches_type(VirtualAccountUpdateResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update_overload_2(self, client: SlashSDK) -> None:
        response = client.virtual_account.with_raw_response.update(
            virtual_account_id="virtualAccountId",
            action="close",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = response.parse()
        assert_matches_type(VirtualAccountUpdateResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update_overload_2(self, client: SlashSDK) -> None:
        with client.virtual_account.with_streaming_response.update(
            virtual_account_id="virtualAccountId",
            action="close",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = response.parse()
            assert_matches_type(VirtualAccountUpdateResponse, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update_overload_2(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `virtual_account_id` but received ''"):
            client.virtual_account.with_raw_response.update(
                virtual_account_id="",
                action="close",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: SlashSDK) -> None:
        virtual_account = client.virtual_account.list()
        assert_matches_type(VirtualAccountListResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: SlashSDK) -> None:
        virtual_account = client.virtual_account.list(
            cursor="cursor",
            filter_account_id="filter:accountId",
            filter_include_closed_accounts="true",
        )
        assert_matches_type(VirtualAccountListResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: SlashSDK) -> None:
        response = client.virtual_account.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = response.parse()
        assert_matches_type(VirtualAccountListResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: SlashSDK) -> None:
        with client.virtual_account.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = response.parse()
            assert_matches_type(VirtualAccountListResponse, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncVirtualAccount:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncSlashSDK) -> None:
        virtual_account = await async_client.virtual_account.create(
            account_id="accountId",
            name="name",
        )
        assert_matches_type(VirtualAccountCreateResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        virtual_account = await async_client.virtual_account.create(
            account_id="accountId",
            name="name",
            commission_details={
                "amount": {"amount_cents": 0},
                "frequency": "monthly",
                "type": "flatFee",
                "start_date": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
        )
        assert_matches_type(VirtualAccountCreateResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.virtual_account.with_raw_response.create(
            account_id="accountId",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = await response.parse()
        assert_matches_type(VirtualAccountCreateResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.virtual_account.with_streaming_response.create(
            account_id="accountId",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = await response.parse()
            assert_matches_type(VirtualAccountCreateResponse, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSlashSDK) -> None:
        virtual_account = await async_client.virtual_account.retrieve(
            "virtualAccountId",
        )
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.virtual_account.with_raw_response.retrieve(
            "virtualAccountId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = await response.parse()
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.virtual_account.with_streaming_response.retrieve(
            "virtualAccountId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = await response.parse()
            assert_matches_type(VirtualAccount, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `virtual_account_id` but received ''"):
            await async_client.virtual_account.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_overload_1(self, async_client: AsyncSlashSDK) -> None:
        virtual_account = await async_client.virtual_account.update(
            virtual_account_id="virtualAccountId",
            action="update",
        )
        assert_matches_type(VirtualAccountUpdateResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params_overload_1(self, async_client: AsyncSlashSDK) -> None:
        virtual_account = await async_client.virtual_account.update(
            virtual_account_id="virtualAccountId",
            action="update",
            commission_details={
                "amount": {"amount_cents": 0},
                "frequency": "monthly",
                "type": "flatFee",
                "start_date": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            name="name",
        )
        assert_matches_type(VirtualAccountUpdateResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update_overload_1(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.virtual_account.with_raw_response.update(
            virtual_account_id="virtualAccountId",
            action="update",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = await response.parse()
        assert_matches_type(VirtualAccountUpdateResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update_overload_1(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.virtual_account.with_streaming_response.update(
            virtual_account_id="virtualAccountId",
            action="update",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = await response.parse()
            assert_matches_type(VirtualAccountUpdateResponse, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update_overload_1(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `virtual_account_id` but received ''"):
            await async_client.virtual_account.with_raw_response.update(
                virtual_account_id="",
                action="update",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_overload_2(self, async_client: AsyncSlashSDK) -> None:
        virtual_account = await async_client.virtual_account.update(
            virtual_account_id="virtualAccountId",
            action="close",
        )
        assert_matches_type(VirtualAccountUpdateResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update_overload_2(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.virtual_account.with_raw_response.update(
            virtual_account_id="virtualAccountId",
            action="close",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = await response.parse()
        assert_matches_type(VirtualAccountUpdateResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update_overload_2(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.virtual_account.with_streaming_response.update(
            virtual_account_id="virtualAccountId",
            action="close",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = await response.parse()
            assert_matches_type(VirtualAccountUpdateResponse, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update_overload_2(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `virtual_account_id` but received ''"):
            await async_client.virtual_account.with_raw_response.update(
                virtual_account_id="",
                action="close",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncSlashSDK) -> None:
        virtual_account = await async_client.virtual_account.list()
        assert_matches_type(VirtualAccountListResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        virtual_account = await async_client.virtual_account.list(
            cursor="cursor",
            filter_account_id="filter:accountId",
            filter_include_closed_accounts="true",
        )
        assert_matches_type(VirtualAccountListResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.virtual_account.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = await response.parse()
        assert_matches_type(VirtualAccountListResponse, virtual_account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.virtual_account.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = await response.parse()
            assert_matches_type(VirtualAccountListResponse, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Transaction", "MerchantData", "MerchantDataLocation", "OriginalCurrency"]


class MerchantDataLocation(BaseModel):
    city: str
    """The city of the merchant."""

    country: str
    """The country of the merchant."""

    state: str
    """The state/province of the merchant."""

    zip: str
    """The ZIP/postal code of the merchant."""


class MerchantData(BaseModel):
    category_code: str = FieldInfo(alias="categoryCode")
    """The merchant's category code (MCC)"""

    description: str
    """The raw description provided by the merchant for the transaction."""

    location: Optional[MerchantDataLocation] = None
    """Location details for the merchant/transaction."""


class OriginalCurrency(BaseModel):
    amount_cents: float = FieldInfo(alias="amountCents")
    """The amount of the transaction in its original currency in cents."""

    code: str
    """The original currency code of the transaction"""

    conversion_rate: float = FieldInfo(alias="conversionRate")
    """
    The conversion rate used to convert the transaction amount from its original
    currency to the account currency. The currency rate is computed at the time of
    the transaction.
    """


class Transaction(BaseModel):
    id: str

    account_id: str = FieldInfo(alias="accountId")
    """The account ID associated with the transaction"""

    account_subtype: Literal["cash", "credit"] = FieldInfo(alias="accountSubtype")
    """
    For charge cards, specifies if the transaction belongs to the cash or credit
    account. For debit accounts, this field is always cash.
    """

    amount_cents: float = FieldInfo(alias="amountCents")
    """The amount of the transaction in cents in USD.

    If the transaction amount is negative, the transaction is a debit. If the
    transaction amount is positive, the transaction is a credit.
    """

    date: str
    """The date in UTC time when the transaction was posted.

    If the transaction is pending or failed, this is the date the transaction was
    created.
    """

    description: str

    detailed_status: Literal[
        "pending",
        "pending_approval",
        "canceled",
        "failed",
        "settled",
        "declined",
        "refund",
        "reversed",
        "returned",
        "dispute",
    ] = FieldInfo(alias="detailedStatus")
    """- `pending_approval`: Used for any transaction type that is pending approval.

    This does not affect the account's available or posted balances.

    - `pending`: Used for any transaction type that has not posted to the account
      yet, but is affecting the account's available balance.
    - `canceled`: Used for any transaction that was canceled.
    - `failed`: Used for any transaction that failed to post to the account.
    - `settled`: Used for any transaction that has posted to the account, except for
      card refunds, which use the `refund` status, and ACH returns, which use the
      `returned` status.
    - `declined`: Used for card transactions that were declined.
    - `refund`: Used for card transactions that were refunded, increasing the
      account's available and posted balances.
    - `reversed`: Used for card transactions that were authorized, but reversed by
      the merchant.
    - `returned`: Used for ACH returns. In the case of an ACH return, there are two
      possibilities: (1) if the ACH settled and then returned, then there will be
      one transaction with status `settled` for the ACH settling, and one
      transaction with status `returned` for the ACH return. Both of these are
      posted transactions. (2) if the ACH returned before settling, then there will
      only be one transaction with status `canceled`.
    - `dispute`: Used for disputed card transactions.
    """

    status: Literal["pending", "posted", "failed"]
    """
    - `pending`: The transaction is pending and has not been posted to the account
      yet (the account's posted balance doesn't reflect the transaction yet, but the
      account's available balance does).
    - `posted`: The transaction has been posted to the account and the account's
      posted balance and available balances both reflect the transaction.
    - `failed`: The transaction failed to post to the account and no longer affects
      the account's available or posted balances.
    """

    authorized_at: Optional[str] = FieldInfo(alias="authorizedAt", default=None)
    """The UTC timestamp of when the transaction was authorized.

    Only exists for card transactions.
    """

    card_id: Optional[str] = FieldInfo(alias="cardId", default=None)
    """The card ID associated with the transaction.

    If the transaction is not associated with a card, this field is not sent.
    """

    decline_reason: Optional[str] = FieldInfo(alias="declineReason", default=None)
    """The reason for the decline.

    Only exists for card transactions with `detailedStatus` = `declined`.
    """

    memo: Optional[str] = None
    """The memo associated with the transaction for wires and achs"""

    merchant_data: Optional[MerchantData] = FieldInfo(alias="merchantData", default=None)
    """
    For card transactions, contains description of the transaction as reported by
    the merchant, merchant category code, and location of the merchant or origin of
    the transaction. For other transactions, this field is undefined.
    """

    merchant_description: Optional[str] = FieldInfo(alias="merchantDescription", default=None)
    """DEPRECATED.

    Use `merchantData.description` instead. For card transactions, the description
    of the transaction as reported by the merchant. For other transactions, this
    field is undefined.
    """

    order_id: Optional[str] = FieldInfo(alias="orderId", default=None)
    """
    The order ID, as reported by the merchant, associated with the specific
    transaction.
    """

    original_currency: Optional[OriginalCurrency] = FieldInfo(alias="originalCurrency", default=None)
    """The original currency of the transaction.

    This is only applicable to transactions. If this field is not sent, the original
    currency is in USD.
    """

    reference_number: Optional[str] = FieldInfo(alias="referenceNumber", default=None)
    """The reference number provided by Visa for this transaction."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Webhook"]


class Webhook(BaseModel):
    id: str

    created_at: str = FieldInfo(alias="createdAt")

    name: str
    """The name of the webhook"""

    url: str
    """The URL that will receive the webhook payload"""

    archived_at: Optional[str] = FieldInfo(alias="archivedAt", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .fdx import (
    FdxResource,
    AsyncFdxResource,
    FdxResourceWithRawResponse,
    AsyncFdxResourceWithRawResponse,
    FdxResourceWithStreamingResponse,
    AsyncFdxResourceWithStreamingResponse,
)
from .pay import (
    PayResource,
    AsyncPayResource,
    PayResourceWithRawResponse,
    AsyncPayResourceWithRawResponse,
    PayResourceWithStreamingResponse,
    AsyncPayResourceWithStreamingResponse,
)
from .card import (
    CardResource,
    AsyncCardResource,
    CardResourceWithRawResponse,
    AsyncCardResourceWithRawResponse,
    CardResourceWithStreamingResponse,
    AsyncCardResourceWithStreamingResponse,
)
from .crypto import (
    CryptoResource,
    AsyncCryptoResource,
    CryptoResourceWithRawResponse,
    AsyncCryptoResourceWithRawResponse,
    CryptoResourceWithStreamingResponse,
    AsyncCryptoResourceWithStreamingResponse,
)
from .oauth2 import (
    Oauth2Resource,
    AsyncOauth2Resource,
    Oauth2ResourceWithRawResponse,
    AsyncOauth2ResourceWithRawResponse,
    Oauth2ResourceWithStreamingResponse,
    AsyncOauth2ResourceWithStreamingResponse,
)
from .account import (
    AccountResource,
    AsyncAccountResource,
    AccountResourceWithRawResponse,
    AsyncAccountResourceWithRawResponse,
    AccountResourceWithStreamingResponse,
    AsyncAccountResourceWithStreamingResponse,
)
from .webhook import (
    WebhookResource,
    AsyncWebhookResource,
    WebhookResourceWithRawResponse,
    AsyncWebhookResourceWithRawResponse,
    WebhookResourceWithStreamingResponse,
    AsyncWebhookResourceWithStreamingResponse,
)
from .merchant import (
    MerchantResource,
    AsyncMerchantResource,
    MerchantResourceWithRawResponse,
    AsyncMerchantResourceWithRawResponse,
    MerchantResourceWithStreamingResponse,
    AsyncMerchantResourceWithStreamingResponse,
)
from .transfer import (
    TransferResource,
    AsyncTransferResource,
    TransferResourceWithRawResponse,
    AsyncTransferResourceWithRawResponse,
    TransferResourceWithStreamingResponse,
    AsyncTransferResourceWithStreamingResponse,
)
from .card_group import (
    CardGroupResource,
    AsyncCardGroupResource,
    CardGroupResourceWithRawResponse,
    AsyncCardGroupResourceWithRawResponse,
    CardGroupResourceWithStreamingResponse,
    AsyncCardGroupResourceWithStreamingResponse,
)
from .well_known import (
    WellKnownResource,
    AsyncWellKnownResource,
    WellKnownResourceWithRawResponse,
    AsyncWellKnownResourceWithRawResponse,
    WellKnownResourceWithStreamingResponse,
    AsyncWellKnownResourceWithStreamingResponse,
)
from .transaction import (
    TransactionResource,
    AsyncTransactionResource,
    TransactionResourceWithRawResponse,
    AsyncTransactionResourceWithRawResponse,
    TransactionResourceWithStreamingResponse,
    AsyncTransactionResourceWithStreamingResponse,
)
from .card_product import (
    CardProductResource,
    AsyncCardProductResource,
    CardProductResourceWithRawResponse,
    AsyncCardProductResourceWithRawResponse,
    CardProductResourceWithStreamingResponse,
    AsyncCardProductResourceWithStreamingResponse,
)
from .legal_entity import (
    LegalEntityResource,
    AsyncLegalEntityResource,
    LegalEntityResourceWithRawResponse,
    AsyncLegalEntityResourceWithRawResponse,
    LegalEntityResourceWithStreamingResponse,
    AsyncLegalEntityResourceWithStreamingResponse,
)
from .slash_handle import (
    SlashHandleResource,
    AsyncSlashHandleResource,
    SlashHandleResourceWithRawResponse,
    AsyncSlashHandleResourceWithRawResponse,
    SlashHandleResourceWithStreamingResponse,
    AsyncSlashHandleResourceWithStreamingResponse,
)
from .virtual_account import (
    VirtualAccountResource,
    AsyncVirtualAccountResource,
    VirtualAccountResourceWithRawResponse,
    AsyncVirtualAccountResourceWithRawResponse,
    VirtualAccountResourceWithStreamingResponse,
    AsyncVirtualAccountResourceWithStreamingResponse,
)
from .developer_account import (
    DeveloperAccountResource,
    AsyncDeveloperAccountResource,
    DeveloperAccountResourceWithRawResponse,
    AsyncDeveloperAccountResourceWithRawResponse,
    DeveloperAccountResourceWithStreamingResponse,
    AsyncDeveloperAccountResourceWithStreamingResponse,
)
from .merchant_category import (
    MerchantCategoryResource,
    AsyncMerchantCategoryResource,
    MerchantCategoryResourceWithRawResponse,
    AsyncMerchantCategoryResourceWithRawResponse,
    MerchantCategoryResourceWithStreamingResponse,
    AsyncMerchantCategoryResourceWithStreamingResponse,
)
from .developer_application import (
    DeveloperApplicationResource,
    AsyncDeveloperApplicationResource,
    DeveloperApplicationResourceWithRawResponse,
    AsyncDeveloperApplicationResourceWithRawResponse,
    DeveloperApplicationResourceWithStreamingResponse,
    AsyncDeveloperApplicationResourceWithStreamingResponse,
)

__all__ = [
    "LegalEntityResource",
    "AsyncLegalEntityResource",
    "LegalEntityResourceWithRawResponse",
    "AsyncLegalEntityResourceWithRawResponse",
    "LegalEntityResourceWithStreamingResponse",
    "AsyncLegalEntityResourceWithStreamingResponse",
    "AccountResource",
    "AsyncAccountResource",
    "AccountResourceWithRawResponse",
    "AsyncAccountResourceWithRawResponse",
    "AccountResourceWithStreamingResponse",
    "AsyncAccountResourceWithStreamingResponse",
    "VirtualAccountResource",
    "AsyncVirtualAccountResource",
    "VirtualAccountResourceWithRawResponse",
    "AsyncVirtualAccountResourceWithRawResponse",
    "VirtualAccountResourceWithStreamingResponse",
    "AsyncVirtualAccountResourceWithStreamingResponse",
    "TransactionResource",
    "AsyncTransactionResource",
    "TransactionResourceWithRawResponse",
    "AsyncTransactionResourceWithRawResponse",
    "TransactionResourceWithStreamingResponse",
    "AsyncTransactionResourceWithStreamingResponse",
    "TransferResource",
    "AsyncTransferResource",
    "TransferResourceWithRawResponse",
    "AsyncTransferResourceWithRawResponse",
    "TransferResourceWithStreamingResponse",
    "AsyncTransferResourceWithStreamingResponse",
    "CardResource",
    "AsyncCardResource",
    "CardResourceWithRawResponse",
    "AsyncCardResourceWithRawResponse",
    "CardResourceWithStreamingResponse",
    "AsyncCardResourceWithStreamingResponse",
    "CardGroupResource",
    "AsyncCardGroupResource",
    "CardGroupResourceWithRawResponse",
    "AsyncCardGroupResourceWithRawResponse",
    "CardGroupResourceWithStreamingResponse",
    "AsyncCardGroupResourceWithStreamingResponse",
    "CardProductResource",
    "AsyncCardProductResource",
    "CardProductResourceWithRawResponse",
    "AsyncCardProductResourceWithRawResponse",
    "CardProductResourceWithStreamingResponse",
    "AsyncCardProductResourceWithStreamingResponse",
    "SlashHandleResource",
    "AsyncSlashHandleResource",
    "SlashHandleResourceWithRawResponse",
    "AsyncSlashHandleResourceWithRawResponse",
    "SlashHandleResourceWithStreamingResponse",
    "AsyncSlashHandleResourceWithStreamingResponse",
    "PayResource",
    "AsyncPayResource",
    "PayResourceWithRawResponse",
    "AsyncPayResourceWithRawResponse",
    "PayResourceWithStreamingResponse",
    "AsyncPayResourceWithStreamingResponse",
    "WebhookResource",
    "AsyncWebhookResource",
    "WebhookResourceWithRawResponse",
    "AsyncWebhookResourceWithRawResponse",
    "WebhookResourceWithStreamingResponse",
    "AsyncWebhookResourceWithStreamingResponse",
    "MerchantResource",
    "AsyncMerchantResource",
    "MerchantResourceWithRawResponse",
    "AsyncMerchantResourceWithRawResponse",
    "MerchantResourceWithStreamingResponse",
    "AsyncMerchantResourceWithStreamingResponse",
    "MerchantCategoryResource",
    "AsyncMerchantCategoryResource",
    "MerchantCategoryResourceWithRawResponse",
    "AsyncMerchantCategoryResourceWithRawResponse",
    "MerchantCategoryResourceWithStreamingResponse",
    "AsyncMerchantCategoryResourceWithStreamingResponse",
    "DeveloperAccountResource",
    "AsyncDeveloperAccountResource",
    "DeveloperAccountResourceWithRawResponse",
    "AsyncDeveloperAccountResourceWithRawResponse",
    "DeveloperAccountResourceWithStreamingResponse",
    "AsyncDeveloperAccountResourceWithStreamingResponse",
    "DeveloperApplicationResource",
    "AsyncDeveloperApplicationResource",
    "DeveloperApplicationResourceWithRawResponse",
    "AsyncDeveloperApplicationResourceWithRawResponse",
    "DeveloperApplicationResourceWithStreamingResponse",
    "AsyncDeveloperApplicationResourceWithStreamingResponse",
    "WellKnownResource",
    "AsyncWellKnownResource",
    "WellKnownResourceWithRawResponse",
    "AsyncWellKnownResourceWithRawResponse",
    "WellKnownResourceWithStreamingResponse",
    "AsyncWellKnownResourceWithStreamingResponse",
    "Oauth2Resource",
    "AsyncOauth2Resource",
    "Oauth2ResourceWithRawResponse",
    "AsyncOauth2ResourceWithRawResponse",
    "Oauth2ResourceWithStreamingResponse",
    "AsyncOauth2ResourceWithStreamingResponse",
    "FdxResource",
    "AsyncFdxResource",
    "FdxResourceWithRawResponse",
    "AsyncFdxResourceWithRawResponse",
    "FdxResourceWithStreamingResponse",
    "AsyncFdxResourceWithStreamingResponse",
    "CryptoResource",
    "AsyncCryptoResource",
    "CryptoResourceWithRawResponse",
    "AsyncCryptoResourceWithRawResponse",
    "CryptoResourceWithStreamingResponse",
    "AsyncCryptoResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from . import loan_account, deposit_account, investment_account
from ..._models import BaseModel
from .line_of_credit_account import LineOfCreditAccount

__all__ = [
    "AccountRetrieveResponse",
    "DepositAccount",
    "DepositAccountDepositAccount",
    "LoanAccount",
    "LoanAccountLoanAccount",
    "LocAccount",
    "LocAccountLocAccount",
    "InvestmentAccount",
    "InvestmentAccountInvestmentAccount",
    "InvestmentAccountInvestmentAccountHolding",
    "InvestmentAccountInvestmentAccountHoldingCurrency",
    "InvestmentAccountInvestmentAccountHoldingFiAttribute",
]


class DepositAccountDepositAccount(deposit_account.DepositAccount):
    available_balance: float = FieldInfo(alias="availableBalance")
    """
    The available balance of the account in the standard unit of the currency (e.g.,
    dollars for USD). Available balance is the balance that is available to
    spend/withdraw.
    """

    current_balance: float = FieldInfo(alias="currentBalance")
    """
    The current balance of the account in the standard unit of the currency (e.g.,
    dollars for USD). Current balance is the balance that is currently in the
    account, including any pending transactions.
    """


class DepositAccount(BaseModel):
    deposit_account: DepositAccountDepositAccount = FieldInfo(alias="depositAccount")
    """A deposit account. For example, a checking, savings or money market account."""


class LoanAccountLoanAccount(loan_account.LoanAccount):
    interest_rate: float = FieldInfo(alias="interestRate")
    """The interest rate on the loan as a floating point number.

    For example, 0.05 for 5%.
    """

    interest_rate_type: Literal["FIXED", "VARIABLE"] = FieldInfo(alias="interestRateType")
    """The type of interest rate on the loan."""

    principal_balance: float = FieldInfo(alias="principalBalance")
    """
    The principal balance of the loan account in the standard unit of the currency
    (e.g., dollars for USD). Principal balance is the amount of the loan that is
    still owed.
    """

    escrow_balance: Optional[float] = FieldInfo(alias="escrowBalance", default=None)
    """
    The escrow balance of the loan account in the standard unit of the currency
    (e.g., dollars for USD). Escrow balance is the amount of money held in escrow to
    pay for property taxes and insurance.
    """

    interest_paid_year_to_date: Optional[float] = FieldInfo(alias="interestPaidYearToDate", default=None)
    """
    The amount of interest paid on the loan year-to-date in the standard unit of the
    currency (e.g., dollars for USD).
    """

    last_payment_amount: Optional[float] = FieldInfo(alias="lastPaymentAmount", default=None)
    """
    The amount of the last payment made on the loan in the standard unit of the
    currency (e.g., dollars for USD).
    """

    last_payment_date: Optional[str] = FieldInfo(alias="lastPaymentDate", default=None)
    """The date the last payment was made."""

    loan_term: Optional[str] = FieldInfo(alias="loanTerm", default=None)
    """The term of the loan in months."""

    maturity_date: Optional[str] = FieldInfo(alias="maturityDate", default=None)
    """The date the loan is scheduled to be paid off."""

    next_payment_amount: Optional[float] = FieldInfo(alias="nextPaymentAmount", default=None)
    """
    The amount of the next payment due on the loan in the standard unit of the
    currency (e.g., dollars for USD).
    """

    next_payment_date: Optional[str] = FieldInfo(alias="nextPaymentDate", default=None)
    """The date the next payment is due."""

    original_principal: Optional[float] = FieldInfo(alias="originalPrincipal", default=None)
    """
    The original principal balance of the loan account in the standard unit of the
    currency (e.g., dollars for USD). Original principal balance is the amount of
    the loan when it was first taken out.
    """

    originating_date: Optional[str] = FieldInfo(alias="originatingDate", default=None)
    """The date the loan was originated."""


class LoanAccount(BaseModel):
    loan_account: LoanAccountLoanAccount = FieldInfo(alias="loanAccount")
    """A loan account. For example, mortgage, student loan or auto loan."""


class LocAccountLocAccount(LineOfCreditAccount):
    available_credit: float = FieldInfo(alias="availableCredit")
    """
    The currently available credit on the account in the standard unit of the
    currency (e.g., dollars for USD).
    """

    current_balance: float = FieldInfo(alias="currentBalance")
    """
    The current balance on the account in the standard unit of the currency (e.g.,
    dollars for USD). This includes any outstanding charges, fees, and interest.
    """

    advances_apr: Optional[float] = FieldInfo(alias="advancesApr", default=None)
    """The annual percentage rate (APR) for cash advances on the account."""

    credit_line: Optional[float] = FieldInfo(alias="creditLine", default=None)
    """
    The total credit line available on the account in the standard unit of the
    currency (e.g., dollars for USD).
    """

    last_payment_amount: Optional[float] = FieldInfo(alias="lastPaymentAmount", default=None)
    """
    The amount of the last payment made on the account in the standard unit of the
    currency (e.g., dollars for USD).
    """

    last_payment_date: Optional[str] = FieldInfo(alias="lastPaymentDate", default=None)
    """The date the last payment was made."""

    last_stmt_balance: Optional[float] = FieldInfo(alias="lastStmtBalance", default=None)
    """
    The balance on the account as of the last statement date in the standard unit of
    the currency (e.g., dollars for USD).
    """

    last_stmt_date: Optional[str] = FieldInfo(alias="lastStmtDate", default=None)
    """The date of the last statement."""

    minimum_payment_amount: Optional[float] = FieldInfo(alias="minimumPaymentAmount", default=None)
    """
    The minimum payment due on the account in the standard unit of the currency
    (e.g., dollars for USD).
    """

    next_payment_amount: Optional[float] = FieldInfo(alias="nextPaymentAmount", default=None)
    """
    The amount of the next payment due on the account in the standard unit of the
    currency (e.g., dollars for USD).
    """

    next_payment_date: Optional[str] = FieldInfo(alias="nextPaymentDate", default=None)
    """The date the next payment is due."""

    past_due_amount: Optional[float] = FieldInfo(alias="pastDueAmount", default=None)
    """
    The amount past due on the account in the standard unit of the currency (e.g.,
    dollars for USD).
    """

    principal_balance: Optional[float] = FieldInfo(alias="principalBalance", default=None)
    """
    The principal balance on the account in the standard unit of the currency (e.g.,
    dollars for USD).
    """

    purchases_apr: Optional[float] = FieldInfo(alias="purchasesApr", default=None)
    """The annual percentage rate (APR) for purchases on the account."""


class LocAccount(BaseModel):
    loc_account: LocAccountLocAccount = FieldInfo(alias="locAccount")
    """A line-of-credit account.

    For example, a credit card or home equity line of credit.
    """


class InvestmentAccountInvestmentAccountHoldingCurrency(BaseModel):
    currency_code: str = FieldInfo(alias="currencyCode")
    """The ISO 4217 currency code."""


class InvestmentAccountInvestmentAccountHoldingFiAttribute(BaseModel):
    name: str

    value: str


class InvestmentAccountInvestmentAccountHolding(BaseModel):
    cash_account: bool = FieldInfo(alias="cashAccount")
    """
    If true, indicates that this holding is used to maintain proceeds from sales,
    dividends, and other cash postings to the investment account.
    """

    market_value: float = FieldInfo(alias="marketValue")
    """
    The market value of the holding in the standard unit of the currency (e.g.,
    dollars for USD).
    """

    currency: Optional[InvestmentAccountInvestmentAccountHoldingCurrency] = None

    current_unit_price: Optional[float] = FieldInfo(alias="currentUnitPrice", default=None)
    """
    The current unit price of the security in the standard unit of the currency
    (e.g., dollars for USD).
    """

    current_unit_price_date: Optional[str] = FieldInfo(alias="currentUnitPriceDate", default=None)
    """The date and time the current unit price was last updated."""

    face_value: Optional[float] = FieldInfo(alias="faceValue", default=None)
    """
    The face value of the holding in the standard unit of the currency (e.g.,
    dollars for USD).
    """

    fi_attributes: Optional[List[InvestmentAccountInvestmentAccountHoldingFiAttribute]] = FieldInfo(
        alias="fiAttributes", default=None
    )

    holding_name: Optional[str] = FieldInfo(alias="holdingName", default=None)
    """The name of the security."""

    holding_sub_type: Optional[Literal["CASH", "MONEYMARKET"]] = FieldInfo(alias="holdingSubType", default=None)

    holding_type: Optional[
        Literal["ANNUITY", "BOND", "CD", "DIGITALASSET", "MUTUALFUND", "OPTION", "OTHER", "STOCK"]
    ] = FieldInfo(alias="holdingType", default=None)
    """The type of the holding."""

    purchased_price: Optional[float] = FieldInfo(alias="purchasedPrice", default=None)
    """
    The price at which the security was purchased in the standard unit of the
    currency (e.g., dollars for USD).
    """

    security_id: Optional[str] = FieldInfo(alias="securityId", default=None)
    """The unique identifier of the security.

    Either securityId and securityIdType or symbol will be provided.
    """

    security_id_type: Optional[
        Literal["CINS", "CMC", "CME", "CUSIP", "ISIN", "ITSA", "NASDAQ", "SEDOL", "SICC", "VALOR", "WKN"]
    ] = FieldInfo(alias="securityIdType", default=None)
    """The type of the security identifier."""

    symbol: Optional[str] = None
    """The symbol of the security.

    Either securityId and securityIdType or symbol will be provided.
    """

    units: Optional[float] = None
    """The number of units of the security held."""


class InvestmentAccountInvestmentAccount(investment_account.InvestmentAccount):
    available_cash_balance: float = FieldInfo(alias="availableCashBalance")
    """
    The amount of cash available in the account across all sub-accounts, including
    sweep funds, in the standard unit of the currency (e.g., dollars for USD).
    """

    current_value: float = FieldInfo(alias="currentValue")
    """
    The total current value of all investments in the standard unit of the currency
    (e.g., dollars for USD).
    """

    balance_as_of: Optional[str] = FieldInfo(alias="balanceAsOf", default=None)
    """The date and time the balance was last updated."""

    holdings: Optional[List[InvestmentAccountInvestmentAccountHolding]] = None
    """The holdings in the account."""


class InvestmentAccount(BaseModel):
    investment_account: InvestmentAccountInvestmentAccount = FieldInfo(alias="investmentAccount")
    """An investment account. For example, a 401K or IRA."""


AccountRetrieveResponse: TypeAlias = Union[DepositAccount, LoanAccount, LocAccount, InvestmentAccount]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from .customers import (
    CustomersResource,
    AsyncCustomersResource,
    CustomersResourceWithRawResponse,
    AsyncCustomersResourceWithRawResponse,
    CustomersResourceWithStreamingResponse,
    AsyncCustomersResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from .accounts.accounts import (
    AccountsResource,
    AsyncAccountsResource,
    AccountsResourceWithRawResponse,
    AsyncAccountsResourceWithRawResponse,
    AccountsResourceWithStreamingResponse,
    AsyncAccountsResourceWithStreamingResponse,
)

__all__ = ["FdxResource", "AsyncFdxResource"]


class FdxResource(SyncAPIResource):
    @cached_property
    def customers(self) -> CustomersResource:
        return CustomersResource(self._client)

    @cached_property
    def accounts(self) -> AccountsResource:
        return AccountsResource(self._client)

    @cached_property
    def with_raw_response(self) -> FdxResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return FdxResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FdxResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return FdxResourceWithStreamingResponse(self)


class AsyncFdxResource(AsyncAPIResource):
    @cached_property
    def customers(self) -> AsyncCustomersResource:
        return AsyncCustomersResource(self._client)

    @cached_property
    def accounts(self) -> AsyncAccountsResource:
        return AsyncAccountsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncFdxResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFdxResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFdxResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncFdxResourceWithStreamingResponse(self)


class FdxResourceWithRawResponse:
    def __init__(self, fdx: FdxResource) -> None:
        self._fdx = fdx

    @cached_property
    def customers(self) -> CustomersResourceWithRawResponse:
        return CustomersResourceWithRawResponse(self._fdx.customers)

    @cached_property
    def accounts(self) -> AccountsResourceWithRawResponse:
        return AccountsResourceWithRawResponse(self._fdx.accounts)


class AsyncFdxResourceWithRawResponse:
    def __init__(self, fdx: AsyncFdxResource) -> None:
        self._fdx = fdx

    @cached_property
    def customers(self) -> AsyncCustomersResourceWithRawResponse:
        return AsyncCustomersResourceWithRawResponse(self._fdx.customers)

    @cached_property
    def accounts(self) -> AsyncAccountsResourceWithRawResponse:
        return AsyncAccountsResourceWithRawResponse(self._fdx.accounts)


class FdxResourceWithStreamingResponse:
    def __init__(self, fdx: FdxResource) -> None:
        self._fdx = fdx

    @cached_property
    def customers(self) -> CustomersResourceWithStreamingResponse:
        return CustomersResourceWithStreamingResponse(self._fdx.customers)

    @cached_property
    def accounts(self) -> AccountsResourceWithStreamingResponse:
        return AccountsResourceWithStreamingResponse(self._fdx.accounts)


class AsyncFdxResourceWithStreamingResponse:
    def __init__(self, fdx: AsyncFdxResource) -> None:
        self._fdx = fdx

    @cached_property
    def customers(self) -> AsyncCustomersResourceWithStreamingResponse:
        return AsyncCustomersResourceWithStreamingResponse(self._fdx.customers)

    @cached_property
    def accounts(self) -> AsyncAccountsResourceWithStreamingResponse:
        return AsyncAccountsResourceWithStreamingResponse(self._fdx.accounts)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .card import Card as Card
from .money import Money as Money
from .account import Account as Account
from .webhook import Webhook as Webhook
from .merchant import Merchant as Merchant
from .card_group import CardGroup as CardGroup
from .card_status import CardStatus as CardStatus
from .money_param import MoneyParam as MoneyParam
from .transaction import Transaction as Transaction
from .slash_handle import SlashHandle as SlashHandle
from .commission_rule import CommissionRule as CommissionRule
from .pay_send_params import PaySendParams as PaySendParams
from .virtual_account import VirtualAccount as VirtualAccount
from .card_list_params import CardListParams as CardListParams
from .pay_send_response import PaySendResponse as PaySendResponse
from .card_create_params import CardCreateParams as CardCreateParams
from .card_list_response import CardListResponse as CardListResponse
from .card_update_params import CardUpdateParams as CardUpdateParams
from .commission_details import CommissionDetails as CommissionDetails
from .account_list_params import AccountListParams as AccountListParams
from .pagination_response import PaginationResponse as PaginationResponse
from .webhook_list_params import WebhookListParams as WebhookListParams
from .card_retrieve_params import CardRetrieveParams as CardRetrieveParams
from .merchant_list_params import MerchantListParams as MerchantListParams
from .account_list_response import AccountListResponse as AccountListResponse
from .virtual_account_model import VirtualAccountModel as VirtualAccountModel
from .webhook_create_params import WebhookCreateParams as WebhookCreateParams
from .webhook_list_response import WebhookListResponse as WebhookListResponse
from .webhook_update_params import WebhookUpdateParams as WebhookUpdateParams
from .card_group_list_params import CardGroupListParams as CardGroupListParams
from .card_group_utilization import CardGroupUtilization as CardGroupUtilization
from .merchant_list_response import MerchantListResponse as MerchantListResponse
from .oauth2_get_token_params import Oauth2GetTokenParams as Oauth2GetTokenParams
from .transaction_list_params import TransactionListParams as TransactionListParams
from .card_group_create_params import CardGroupCreateParams as CardGroupCreateParams
from .card_group_list_response import CardGroupListResponse as CardGroupListResponse
from .card_group_update_params import CardGroupUpdateParams as CardGroupUpdateParams
from .card_product_list_params import CardProductListParams as CardProductListParams
from .commission_details_param import CommissionDetailsParam as CommissionDetailsParam
from .slash_handle_list_params import SlashHandleListParams as SlashHandleListParams
from .oauth2_get_token_response import Oauth2GetTokenResponse as Oauth2GetTokenResponse
from .transaction_list_response import TransactionListResponse as TransactionListResponse
from .card_product_list_response import CardProductListResponse as CardProductListResponse
from .developer_application_data import DeveloperApplicationData as DeveloperApplicationData
from .developer_application_type import DeveloperApplicationType as DeveloperApplicationType
from .legal_entity_list_response import LegalEntityListResponse as LegalEntityListResponse
from .slash_handle_list_response import SlashHandleListResponse as SlashHandleListResponse
from .developer_application_model import DeveloperApplicationModel as DeveloperApplicationModel
from .virtual_account_list_params import VirtualAccountListParams as VirtualAccountListParams
from .crypto_create_offramp_params import CryptoCreateOfframpParams as CryptoCreateOfframpParams
from .transaction_aggregate_params import TransactionAggregateParams as TransactionAggregateParams
from .merchant_category_list_params import MerchantCategoryListParams as MerchantCategoryListParams
from .virtual_account_create_params import VirtualAccountCreateParams as VirtualAccountCreateParams
from .virtual_account_list_response import VirtualAccountListResponse as VirtualAccountListResponse
from .virtual_account_update_params import VirtualAccountUpdateParams as VirtualAccountUpdateParams
from .crypto_create_offramp_response import CryptoCreateOfframpResponse as CryptoCreateOfframpResponse
from .transaction_aggregate_response import TransactionAggregateResponse as TransactionAggregateResponse
from .transaction_update_note_params import TransactionUpdateNoteParams as TransactionUpdateNoteParams
from .merchant_category_list_response import MerchantCategoryListResponse as MerchantCategoryListResponse
from .virtual_account_create_response import VirtualAccountCreateResponse as VirtualAccountCreateResponse
from .virtual_account_update_response import VirtualAccountUpdateResponse as VirtualAccountUpdateResponse
from .developer_application_data_param import DeveloperApplicationDataParam as DeveloperApplicationDataParam
from .transaction_update_note_response import TransactionUpdateNoteResponse as TransactionUpdateNoteResponse
from .account_retrieve_balance_response import AccountRetrieveBalanceResponse as AccountRetrieveBalanceResponse
from .developer_application_update_params import DeveloperApplicationUpdateParams as DeveloperApplicationUpdateParams
from .transaction_retrieve_fee_details_response import (
    TransactionRetrieveFeeDetailsResponse as TransactionRetrieveFeeDetailsResponse,
)
from .developer_account_create_application_params import (
    DeveloperAccountCreateApplicationParams as DeveloperAccountCreateApplicationParams,
)
from .transfer_create_virtual_account_transfer_params import (
    TransferCreateVirtualAccountTransferParams as TransferCreateVirtualAccountTransferParams,
)
from .transfer_create_virtual_account_transfer_response import (
    TransferCreateVirtualAccountTransferResponse as TransferCreateVirtualAccountTransferResponse,
)
from .well_known_retrieve_openid_configuration_response import (
    WellKnownRetrieveOpenidConfigurationResponse as WellKnownRetrieveOpenidConfigurationResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .commission_rule import CommissionRule
from .virtual_account_model import VirtualAccountModel

__all__ = ["VirtualAccountCreateResponse"]


class VirtualAccountCreateResponse(BaseModel):
    virtual_account: VirtualAccountModel = FieldInfo(alias="virtualAccount")
    """The virtual account object"""

    commission_rule: Optional[CommissionRule] = FieldInfo(alias="commissionRule", default=None)
    """The virtual account commission rule object"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .money import Money
from .._models import BaseModel

__all__ = ["AccountRetrieveBalanceResponse", "Balance"]


class Balance(BaseModel):
    account_id: str = FieldInfo(alias="accountId")
    """The ID of the account associated with the balance."""

    available: Money
    """Represents a monetary value"""

    posted: Money
    """Represents a monetary value"""

    timestamp: datetime
    """The UTC timestamp associated with the balance.

    Since the balance can change rapidly, the timestamp is included to indicate the
    time when the balance was computed.
    """

    type: Literal["cash", "credit", "debit"]
    """The type of balance.

    - "cash" represents the excess balance not used as collateral for the charge
      card. Not applicable to debit accounts.
    - "credit" represents the balance that can be spent on the charge card. Not
      applicable to debit accounts.
    - "debit" represents the balance that can be spent or withdrawn. Not applicable
      to charge card accounts.
    """


class AccountRetrieveBalanceResponse(BaseModel):
    balances: List[Balance]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo
from .money_param import MoneyParam

__all__ = ["CommissionDetailsParam", "UnionMember0", "UnionMember1"]


class UnionMember0(TypedDict, total=False):
    amount: Required[MoneyParam]
    """
    The amount of money to be diverted from the virtual account if the type of rule
    is flat_fee.
    """

    frequency: Required[Literal["monthly", "yearly"]]
    """The frequency from the start date that the rule will be applied.

    E.g. a start date of November 26, 2024 with a frequency of "monthly" will have
    the rule applied on December 26, 2024. If the frequency was "yearly", the rule
    would be applied again on November 26, 2025.
    """

    type: Required[Literal["flatFee"]]
    """The type of the commission rule.

    A type of flat_fee will take a flat fee from the virtual account and transfer it
    to the primary account with the frequency of the collection defined as monthly
    or yearly.
    """

    start_date: Annotated[Union[str, datetime], PropertyInfo(alias="startDate", format="iso8601")]
    """The date that the rule will begin to take effect."""


class UnionMember1(TypedDict, total=False):
    take_rate: Required[Annotated[float, PropertyInfo(alias="takeRate")]]
    """
    The percentage of funds to divert from the virtual account to the primary
    account.
    """

    type: Required[Literal["takeRate"]]
    """The type of the commission rule.

    A type of take_rate will take a percent of the virtual account's funds and
    divert them to the primary account.
    """


CommissionDetailsParam: TypeAlias = Union[UnionMember0, UnionMember1]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["WebhookUpdateParams"]


class WebhookUpdateParams(TypedDict, total=False):
    status: Required[Literal["archived"]]

    reason: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["MoneyParam"]


class MoneyParam(TypedDict, total=False):
    amount_cents: Required[Annotated[int, PropertyInfo(alias="amountCents")]]
    """The amount in cents"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["VirtualAccountModel"]


class VirtualAccountModel(BaseModel):
    id: str
    """The ID of the virtual account."""

    account_id: str = FieldInfo(alias="accountId")
    """The account ID of the Slash account the virtual account is associated with."""

    account_type: Literal["default", "primary"] = FieldInfo(alias="accountType")
    """The account type of the virtual account.

    Since virtual accounts and Slash accounts exist at the same hierarchy level, an
    account type of 'primary' means it is the main Slash account, while an account
    type of 'default' means it is a virtual account under the main account.
    """

    name: str
    """The name of the virtual account."""

    account_number: Optional[str] = FieldInfo(alias="accountNumber", default=None)
    """The account number of the virtual account."""

    closed_at: Optional[str] = FieldInfo(alias="closedAt", default=None)
    """
    The date the virtual account was closed; will be null if account was never
    closed.
    """

    routing_number: Optional[str] = FieldInfo(alias="routingNumber", default=None)
    """The routing number of the virtual account."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .commission_details import CommissionDetails

__all__ = ["CommissionRule"]


class CommissionRule(BaseModel):
    id: str
    """The ID of the commission rule"""

    commission_details: CommissionDetails = FieldInfo(alias="commissionDetails")
    """Describes details of the commission rule."""

    virtual_account_id: str = FieldInfo(alias="virtualAccountId")
    """The ID of the virtual account the commission rule is applied to"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import transaction_list_params, transaction_aggregate_params, transaction_update_note_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.transaction import Transaction
from ..types.transaction_list_response import TransactionListResponse
from ..types.transaction_aggregate_response import TransactionAggregateResponse
from ..types.transaction_update_note_response import TransactionUpdateNoteResponse
from ..types.transaction_retrieve_fee_details_response import TransactionRetrieveFeeDetailsResponse

__all__ = ["TransactionResource", "AsyncTransactionResource"]


class TransactionResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TransactionResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return TransactionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TransactionResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return TransactionResourceWithStreamingResponse(self)

    def retrieve(
        self,
        transaction_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Transaction:
        """
        Fetch details for a single transaction by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not transaction_id:
            raise ValueError(f"Expected a non-empty value for `transaction_id` but received {transaction_id!r}")
        return self._get(
            f"/transaction/{transaction_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Transaction,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        cursor: str | NotGiven = NOT_GIVEN,
        filter_account_id: str | NotGiven = NOT_GIVEN,
        filter_card_id: str | NotGiven = NOT_GIVEN,
        filter_detailed_status: Literal[
            "pending", "canceled", "failed", "settled", "declined", "refund", "reversed", "returned", "dispute"
        ]
        | NotGiven = NOT_GIVEN,
        filter_from_authorized_at: str | NotGiven = NOT_GIVEN,
        filter_from_date: str | NotGiven = NOT_GIVEN,
        filter_legal_entity_id: str | NotGiven = NOT_GIVEN,
        filter_status: Literal["pending", "posted", "failed"] | NotGiven = NOT_GIVEN,
        filter_to_authorized_at: str | NotGiven = NOT_GIVEN,
        filter_to_date: str | NotGiven = NOT_GIVEN,
        filter_virtual_account_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransactionListResponse:
        """
        Get all transactions

        Args:
          account_id: Use filter:accountId to filter by account ID

          cursor: A cursor string to fetch the next page of results

          filter_account_id: Pass in an account ID to filter transactions by account ID. This will return all
              transactions that match the account ID passed in.

          filter_card_id: Filter transactions by cardId

          filter_detailed_status: Filter transactions by detailed status

          filter_from_authorized_at: Pass in a unix timestamp in milliseconds to filter transactions by authorization
              time. This will return all transactions that are authorized on or after the date
              passed in.

          filter_from_date: Pass in a unix timestamp in milliseconds to filter transactions by date. This
              will return all transactions that occurred on or after the date passed in.

          filter_legal_entity_id: Pass in a legal entity ID to filter transactions by accounts under a specific
              legal entity.

          filter_status: Filter transactions by status

          filter_to_authorized_at: Pass in a unix timestamp in milliseconds to filter transactions by authorization
              time. This will return all transactions that are authorized on or before the
              date passed in.

          filter_to_date: Pass in a unix timestamp in milliseconds to filter transactions by date. This
              will return all transactions that occurred on or before the date passed in.

          filter_virtual_account_id: Pass in a virtual account ID to filter transactions by virtual account ID. This
              will return all transactions that match the virtual account ID passed in.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/transaction",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "cursor": cursor,
                        "filter_account_id": filter_account_id,
                        "filter_card_id": filter_card_id,
                        "filter_detailed_status": filter_detailed_status,
                        "filter_from_authorized_at": filter_from_authorized_at,
                        "filter_from_date": filter_from_date,
                        "filter_legal_entity_id": filter_legal_entity_id,
                        "filter_status": filter_status,
                        "filter_to_authorized_at": filter_to_authorized_at,
                        "filter_to_date": filter_to_date,
                        "filter_virtual_account_id": filter_virtual_account_id,
                    },
                    transaction_list_params.TransactionListParams,
                ),
            ),
            cast_to=TransactionListResponse,
        )

    def aggregate(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        filter_account_id: str | NotGiven = NOT_GIVEN,
        filter_card_id: str | NotGiven = NOT_GIVEN,
        filter_detailed_status: Literal[
            "pending", "canceled", "failed", "settled", "declined", "refund", "reversed", "returned", "dispute"
        ]
        | NotGiven = NOT_GIVEN,
        filter_from_authorized_at: str | NotGiven = NOT_GIVEN,
        filter_from_date: str | NotGiven = NOT_GIVEN,
        filter_legal_entity_id: str | NotGiven = NOT_GIVEN,
        filter_status: Literal["pending", "posted", "failed"] | NotGiven = NOT_GIVEN,
        filter_to_authorized_at: str | NotGiven = NOT_GIVEN,
        filter_to_date: str | NotGiven = NOT_GIVEN,
        filter_virtual_account_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransactionAggregateResponse:
        """
        Get transaction aggregations

        Args:
          account_id: Use filter:accountId to filter by account ID

          filter_account_id: Pass in an account ID to filter transactions by account ID. This will return all
              transactions that match the account ID passed in.

          filter_card_id: Filter transactions by cardId

          filter_detailed_status: Filter transactions by detailed status

          filter_from_authorized_at: Pass in a unix timestamp in milliseconds to filter transactions by authorization
              time. This will return all transactions that are authorized on or after the date
              passed in.

          filter_from_date: Pass in a unix timestamp in milliseconds to filter transactions by date. This
              will return all transactions that occurred on or after the date passed in.

          filter_legal_entity_id: Pass in a legal entity ID to filter transactions by accounts under a specific
              legal entity.

          filter_status: Filter transactions by status

          filter_to_authorized_at: Pass in a unix timestamp in milliseconds to filter transactions by authorization
              time. This will return all transactions that are authorized on or before the
              date passed in.

          filter_to_date: Pass in a unix timestamp in milliseconds to filter transactions by date. This
              will return all transactions that occurred on or before the date passed in.

          filter_virtual_account_id: Pass in a virtual account ID to filter transactions by virtual account ID. This
              will return all transactions that match the virtual account ID passed in.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/transaction/aggregation",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "filter_account_id": filter_account_id,
                        "filter_card_id": filter_card_id,
                        "filter_detailed_status": filter_detailed_status,
                        "filter_from_authorized_at": filter_from_authorized_at,
                        "filter_from_date": filter_from_date,
                        "filter_legal_entity_id": filter_legal_entity_id,
                        "filter_status": filter_status,
                        "filter_to_authorized_at": filter_to_authorized_at,
                        "filter_to_date": filter_to_date,
                        "filter_virtual_account_id": filter_virtual_account_id,
                    },
                    transaction_aggregate_params.TransactionAggregateParams,
                ),
            ),
            cast_to=TransactionAggregateResponse,
        )

    def retrieve_fee_details(
        self,
        transaction_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransactionRetrieveFeeDetailsResponse:
        """
        Fetch breakdown of a fee transaction

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not transaction_id:
            raise ValueError(f"Expected a non-empty value for `transaction_id` but received {transaction_id!r}")
        return self._get(
            f"/transaction/{transaction_id}/fee-details",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionRetrieveFeeDetailsResponse,
        )

    def update_note(
        self,
        transaction_id: str,
        *,
        note: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransactionUpdateNoteResponse:
        """
        Update note for a transaction

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not transaction_id:
            raise ValueError(f"Expected a non-empty value for `transaction_id` but received {transaction_id!r}")
        return self._patch(
            f"/transaction/{transaction_id}/note",
            body=maybe_transform({"note": note}, transaction_update_note_params.TransactionUpdateNoteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionUpdateNoteResponse,
        )


class AsyncTransactionResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTransactionResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTransactionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTransactionResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/slashfi/slash-sdk-python#with_streaming_response
        """
        return AsyncTransactionResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        transaction_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Transaction:
        """
        Fetch details for a single transaction by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not transaction_id:
            raise ValueError(f"Expected a non-empty value for `transaction_id` but received {transaction_id!r}")
        return await self._get(
            f"/transaction/{transaction_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Transaction,
        )

    async def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        cursor: str | NotGiven = NOT_GIVEN,
        filter_account_id: str | NotGiven = NOT_GIVEN,
        filter_card_id: str | NotGiven = NOT_GIVEN,
        filter_detailed_status: Literal[
            "pending", "canceled", "failed", "settled", "declined", "refund", "reversed", "returned", "dispute"
        ]
        | NotGiven = NOT_GIVEN,
        filter_from_authorized_at: str | NotGiven = NOT_GIVEN,
        filter_from_date: str | NotGiven = NOT_GIVEN,
        filter_legal_entity_id: str | NotGiven = NOT_GIVEN,
        filter_status: Literal["pending", "posted", "failed"] | NotGiven = NOT_GIVEN,
        filter_to_authorized_at: str | NotGiven = NOT_GIVEN,
        filter_to_date: str | NotGiven = NOT_GIVEN,
        filter_virtual_account_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransactionListResponse:
        """
        Get all transactions

        Args:
          account_id: Use filter:accountId to filter by account ID

          cursor: A cursor string to fetch the next page of results

          filter_account_id: Pass in an account ID to filter transactions by account ID. This will return all
              transactions that match the account ID passed in.

          filter_card_id: Filter transactions by cardId

          filter_detailed_status: Filter transactions by detailed status

          filter_from_authorized_at: Pass in a unix timestamp in milliseconds to filter transactions by authorization
              time. This will return all transactions that are authorized on or after the date
              passed in.

          filter_from_date: Pass in a unix timestamp in milliseconds to filter transactions by date. This
              will return all transactions that occurred on or after the date passed in.

          filter_legal_entity_id: Pass in a legal entity ID to filter transactions by accounts under a specific
              legal entity.

          filter_status: Filter transactions by status

          filter_to_authorized_at: Pass in a unix timestamp in milliseconds to filter transactions by authorization
              time. This will return all transactions that are authorized on or before the
              date passed in.

          filter_to_date: Pass in a unix timestamp in milliseconds to filter transactions by date. This
              will return all transactions that occurred on or before the date passed in.

          filter_virtual_account_id: Pass in a virtual account ID to filter transactions by virtual account ID. This
              will return all transactions that match the virtual account ID passed in.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/transaction",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "account_id": account_id,
                        "cursor": cursor,
                        "filter_account_id": filter_account_id,
                        "filter_card_id": filter_card_id,
                        "filter_detailed_status": filter_detailed_status,
                        "filter_from_authorized_at": filter_from_authorized_at,
                        "filter_from_date": filter_from_date,
                        "filter_legal_entity_id": filter_legal_entity_id,
                        "filter_status": filter_status,
                        "filter_to_authorized_at": filter_to_authorized_at,
                        "filter_to_date": filter_to_date,
                        "filter_virtual_account_id": filter_virtual_account_id,
                    },
                    transaction_list_params.TransactionListParams,
                ),
            ),
            cast_to=TransactionListResponse,
        )

    async def aggregate(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        filter_account_id: str | NotGiven = NOT_GIVEN,
        filter_card_id: str | NotGiven = NOT_GIVEN,
        filter_detailed_status: Literal[
            "pending", "canceled", "failed", "settled", "declined", "refund", "reversed", "returned", "dispute"
        ]
        | NotGiven = NOT_GIVEN,
        filter_from_authorized_at: str | NotGiven = NOT_GIVEN,
        filter_from_date: str | NotGiven = NOT_GIVEN,
        filter_legal_entity_id: str | NotGiven = NOT_GIVEN,
        filter_status: Literal["pending", "posted", "failed"] | NotGiven = NOT_GIVEN,
        filter_to_authorized_at: str | NotGiven = NOT_GIVEN,
        filter_to_date: str | NotGiven = NOT_GIVEN,
        filter_virtual_account_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransactionAggregateResponse:
        """
        Get transaction aggregations

        Args:
          account_id: Use filter:accountId to filter by account ID

          filter_account_id: Pass in an account ID to filter transactions by account ID. This will return all
              transactions that match the account ID passed in.

          filter_card_id: Filter transactions by cardId

          filter_detailed_status: Filter transactions by detailed status

          filter_from_authorized_at: Pass in a unix timestamp in milliseconds to filter transactions by authorization
              time. This will return all transactions that are authorized on or after the date
              passed in.

          filter_from_date: Pass in a unix timestamp in milliseconds to filter transactions by date. This
              will return all transactions that occurred on or after the date passed in.

          filter_legal_entity_id: Pass in a legal entity ID to filter transactions by accounts under a specific
              legal entity.

          filter_status: Filter transactions by status

          filter_to_authorized_at: Pass in a unix timestamp in milliseconds to filter transactions by authorization
              time. This will return all transactions that are authorized on or before the
              date passed in.

          filter_to_date: Pass in a unix timestamp in milliseconds to filter transactions by date. This
              will return all transactions that occurred on or before the date passed in.

          filter_virtual_account_id: Pass in a virtual account ID to filter transactions by virtual account ID. This
              will return all transactions that match the virtual account ID passed in.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/transaction/aggregation",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "account_id": account_id,
                        "filter_account_id": filter_account_id,
                        "filter_card_id": filter_card_id,
                        "filter_detailed_status": filter_detailed_status,
                        "filter_from_authorized_at": filter_from_authorized_at,
                        "filter_from_date": filter_from_date,
                        "filter_legal_entity_id": filter_legal_entity_id,
                        "filter_status": filter_status,
                        "filter_to_authorized_at": filter_to_authorized_at,
                        "filter_to_date": filter_to_date,
                        "filter_virtual_account_id": filter_virtual_account_id,
                    },
                    transaction_aggregate_params.TransactionAggregateParams,
                ),
            ),
            cast_to=TransactionAggregateResponse,
        )

    async def retrieve_fee_details(
        self,
        transaction_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransactionRetrieveFeeDetailsResponse:
        """
        Fetch breakdown of a fee transaction

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not transaction_id:
            raise ValueError(f"Expected a non-empty value for `transaction_id` but received {transaction_id!r}")
        return await self._get(
            f"/transaction/{transaction_id}/fee-details",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionRetrieveFeeDetailsResponse,
        )

    async def update_note(
        self,
        transaction_id: str,
        *,
        note: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransactionUpdateNoteResponse:
        """
        Update note for a transaction

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not transaction_id:
            raise ValueError(f"Expected a non-empty value for `transaction_id` but received {transaction_id!r}")
        return await self._patch(
            f"/transaction/{transaction_id}/note",
            body=await async_maybe_transform(
                {"note": note}, transaction_update_note_params.TransactionUpdateNoteParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionUpdateNoteResponse,
        )


class TransactionResourceWithRawResponse:
    def __init__(self, transaction: TransactionResource) -> None:
        self._transaction = transaction

        self.retrieve = to_raw_response_wrapper(
            transaction.retrieve,
        )
        self.list = to_raw_response_wrapper(
            transaction.list,
        )
        self.aggregate = to_raw_response_wrapper(
            transaction.aggregate,
        )
        self.retrieve_fee_details = to_raw_response_wrapper(
            transaction.retrieve_fee_details,
        )
        self.update_note = to_raw_response_wrapper(
            transaction.update_note,
        )


class AsyncTransactionResourceWithRawResponse:
    def __init__(self, transaction: AsyncTransactionResource) -> None:
        self._transaction = transaction

        self.retrieve = async_to_raw_response_wrapper(
            transaction.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            transaction.list,
        )
        self.aggregate = async_to_raw_response_wrapper(
            transaction.aggregate,
        )
        self.retrieve_fee_details = async_to_raw_response_wrapper(
            transaction.retrieve_fee_details,
        )
        self.update_note = async_to_raw_response_wrapper(
            transaction.update_note,
        )


class TransactionResourceWithStreamingResponse:
    def __init__(self, transaction: TransactionResource) -> None:
        self._transaction = transaction

        self.retrieve = to_streamed_response_wrapper(
            transaction.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            transaction.list,
        )
        self.aggregate = to_streamed_response_wrapper(
            transaction.aggregate,
        )
        self.retrieve_fee_details = to_streamed_response_wrapper(
            transaction.retrieve_fee_details,
        )
        self.update_note = to_streamed_response_wrapper(
            transaction.update_note,
        )


class AsyncTransactionResourceWithStreamingResponse:
    def __init__(self, transaction: AsyncTransactionResource) -> None:
        self._transaction = transaction

        self.retrieve = async_to_streamed_response_wrapper(
            transaction.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            transaction.list,
        )
        self.aggregate = async_to_streamed_response_wrapper(
            transaction.aggregate,
        )
        self.retrieve_fee_details = async_to_streamed_response_wrapper(
            transaction.retrieve_fee_details,
        )
        self.update_note = async_to_streamed_response_wrapper(
            transaction.update_note,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["AccountRetrievePaymentNetworksResponse", "PaymentNetwork", "Page"]


class PaymentNetwork(BaseModel):
    bank_id: str = FieldInfo(alias="bankId")
    """Bank identifier used by the payment network ie. Routing Number"""

    identifier: str
    """The number used to identify the account within the payment network."""

    transfer_in: bool = FieldInfo(alias="transferIn")
    """Indicates if the account can receive transfers from the payment network.

    If the account is closed or locked, this will be false.
    """

    transfer_out: bool = FieldInfo(alias="transferOut")
    """Indicates if the account can send transfers to the payment network.

    If the account is closed or locked, this will be false.
    """

    type: Literal["US_ACH", "US_FEDWIRE", "US_CHIPS", "US_RTP", "CA_ACSS", "CA_LVTS"]


class Page(BaseModel):
    next_offset: Optional[str] = FieldInfo(alias="nextOffset", default=None)


class AccountRetrievePaymentNetworksResponse(BaseModel):
    payment_networks: List[PaymentNetwork] = FieldInfo(alias="paymentNetworks")

    page: Optional[Page] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from ..card.spending_constraint import SpendingConstraint

__all__ = ["CardGroup"]


class CardGroup(BaseModel):
    id: str

    name: str

    spending_constraint: Optional[SpendingConstraint] = FieldInfo(alias="spendingConstraint", default=None)
    """A constraint that can be applied to a CardGroupSpendingRule"""

    virtual_account_id: Optional[str] = FieldInfo(alias="virtualAccountId", default=None)
    """The ID of the virtual account that the card group is associated with"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CardListParams"]


class CardListParams(TypedDict, total=False):
    cursor: str
    """A cursor string to fetch the next page of results"""

    filter_account_id: Annotated[str, PropertyInfo(alias="filter:accountId")]
    """Pass in an account ID to filter for cards under a specific account."""

    filter_card_group_id: Annotated[str, PropertyInfo(alias="filter:cardGroupId")]
    """
    Pass in a card group ID, This will return all cards that belong to the card
    group ID passed in. Cannot be combined with filter:cardGroupName.
    """

    filter_card_group_name: Annotated[str, PropertyInfo(alias="filter:cardGroupName")]
    """
    Pass in a card group name, This will return all cards that belong to the card
    group name passed in. Cannot be combined with filter:cardGroupId.
    """

    filter_legal_entity_id: Annotated[str, PropertyInfo(alias="filter:legalEntityId")]
    """
    Pass in a legal entity ID to filter for cards in accounts under a specific legal
    entity.
    """

    filter_status: Annotated[Literal["active", "paused", "closed", "inactive"], PropertyInfo(alias="filter:status")]
    """Returns all cards matching the status passed in."""

    filter_virtual_account_id: Annotated[str, PropertyInfo(alias="filter:virtualAccountId")]
    """
    Pass in a virtual account ID to filter for cards under a specific virtual
    account.
    """

    sort: Literal["createdAt", "name"]
    """Sorts card by creation date or name."""

    sort_direction: Annotated[Literal["ASC", "DESC"], PropertyInfo(alias="sortDirection")]
    """The direction to apply the sort filter by. Default ASC."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["SlashHandle"]


class SlashHandle(BaseModel):
    id: str
    """The id of the SlashHandle"""

    account_id: str = FieldInfo(alias="accountId")
    """The ID of the account that will send and receive funds for this entity"""

    name: str
    """The display name of the entity"""

    slash_handle: str = FieldInfo(alias="slashHandle")
    """The username that others can use to send money to this entity"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Account"]


class Account(BaseModel):
    id: str

    account_number: str = FieldInfo(alias="accountNumber")

    balances: List[Literal["debit", "cash", "credit"]]

    created_at: datetime = FieldInfo(alias="createdAt")

    name: str

    routing_number: str = FieldInfo(alias="routingNumber")

    status: Literal["open", "closed"]

    type: Literal["debit", "charge_card"]
    """The type of account.

    - "debit" represents a debit card account - "charge_card" represents a charge
      card account
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["CryptoCreateOfframpResponse", "Address", "Wallet"]


class Address(BaseModel):
    id: str

    address: str

    chain: Literal["ethereum", "avalanche", "base", "optimism", "polygon", "arbitrum", "solana", "stellar", "tron"]

    currency: Literal["usdc", "usdt"]

    status: Literal["pending_active", "active", "deactivated"]

    timestamp: datetime

    wallet_id: str = FieldInfo(alias="walletId")

    blockchain_memo: Optional[str] = FieldInfo(alias="blockchainMemo", default=None)

    method: Optional[Literal["wire", "ach"]] = None


class Wallet(BaseModel):
    id: str

    slash_account_group_id: str = FieldInfo(alias="slashAccountGroupId")

    status: Literal["pending_active", "active", "deactivated"]

    subaccount_id: str = FieldInfo(alias="subaccountId")

    timestamp: datetime

    type: Literal["off_ramp"]


class CryptoCreateOfframpResponse(BaseModel):
    addresses: List[Address]

    wallet: Wallet



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from slash_sdk import SlashSDK, AsyncSlashSDK
from tests.utils import assert_matches_type
from slash_sdk.types import Account, AccountListResponse, AccountRetrieveBalanceResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAccount:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: SlashSDK) -> None:
        account = client.account.retrieve(
            "accountId",
        )
        assert_matches_type(Account, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: SlashSDK) -> None:
        response = client.account.with_raw_response.retrieve(
            "accountId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = response.parse()
        assert_matches_type(Account, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: SlashSDK) -> None:
        with client.account.with_streaming_response.retrieve(
            "accountId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = response.parse()
            assert_matches_type(Account, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.account.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: SlashSDK) -> None:
        account = client.account.list()
        assert_matches_type(AccountListResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: SlashSDK) -> None:
        account = client.account.list(
            filter_legal_entity_id="filter:legalEntityId",
        )
        assert_matches_type(AccountListResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: SlashSDK) -> None:
        response = client.account.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = response.parse()
        assert_matches_type(AccountListResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: SlashSDK) -> None:
        with client.account.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = response.parse()
            assert_matches_type(AccountListResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_balance(self, client: SlashSDK) -> None:
        account = client.account.retrieve_balance(
            "accountId",
        )
        assert_matches_type(AccountRetrieveBalanceResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve_balance(self, client: SlashSDK) -> None:
        response = client.account.with_raw_response.retrieve_balance(
            "accountId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = response.parse()
        assert_matches_type(AccountRetrieveBalanceResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve_balance(self, client: SlashSDK) -> None:
        with client.account.with_streaming_response.retrieve_balance(
            "accountId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = response.parse()
            assert_matches_type(AccountRetrieveBalanceResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve_balance(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            client.account.with_raw_response.retrieve_balance(
                "",
            )


class TestAsyncAccount:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSlashSDK) -> None:
        account = await async_client.account.retrieve(
            "accountId",
        )
        assert_matches_type(Account, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.account.with_raw_response.retrieve(
            "accountId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = await response.parse()
        assert_matches_type(Account, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.account.with_streaming_response.retrieve(
            "accountId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = await response.parse()
            assert_matches_type(Account, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.account.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncSlashSDK) -> None:
        account = await async_client.account.list()
        assert_matches_type(AccountListResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        account = await async_client.account.list(
            filter_legal_entity_id="filter:legalEntityId",
        )
        assert_matches_type(AccountListResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.account.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = await response.parse()
        assert_matches_type(AccountListResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.account.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = await response.parse()
            assert_matches_type(AccountListResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_balance(self, async_client: AsyncSlashSDK) -> None:
        account = await async_client.account.retrieve_balance(
            "accountId",
        )
        assert_matches_type(AccountRetrieveBalanceResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve_balance(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.account.with_raw_response.retrieve_balance(
            "accountId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        account = await response.parse()
        assert_matches_type(AccountRetrieveBalanceResponse, account, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve_balance(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.account.with_streaming_response.retrieve_balance(
            "accountId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            account = await response.parse()
            assert_matches_type(AccountRetrieveBalanceResponse, account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve_balance(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_id` but received ''"):
            await async_client.account.with_raw_response.retrieve_balance(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .transaction import Transaction
from .pagination_response import PaginationResponse

__all__ = ["TransactionListResponse"]


class TransactionListResponse(BaseModel):
    items: List[Transaction]

    metadata: Optional[PaginationResponse] = None
    """Response sent when requesting a list of data"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .card import Card as Card
from .restriction import Restriction as Restriction
from .spending_constraint import SpendingConstraint as SpendingConstraint
from .spending_constraint_param import SpendingConstraintParam as SpendingConstraintParam
from .spending_constraint_update_full_params import (
    SpendingConstraintUpdateFullParams as SpendingConstraintUpdateFullParams,
)
from .spending_constraint_update_partial_params import (
    SpendingConstraintUpdatePartialParams as SpendingConstraintUpdatePartialParams,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["MerchantCategoryListParams"]


class MerchantCategoryListParams(TypedDict, total=False):
    cursor: str
    """A cursor string to fetch the next page of results"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from slash_sdk import SlashSDK, AsyncSlashSDK
from tests.utils import assert_matches_type
from slash_sdk.types import (
    CardGroupUtilization,
    CardGroupListResponse,
)
from slash_sdk.types.card_group import CardGroup

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCardGroup:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: SlashSDK) -> None:
        card_group = client.card_group.create(
            name="name",
        )
        assert_matches_type(CardGroup, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: SlashSDK) -> None:
        card_group = client.card_group.create(
            name="name",
            spending_constraint={
                "country_rule": {
                    "countries": ["string"],
                    "restriction": "allowlist",
                },
                "merchant_category_code_rule": {
                    "merchant_category_codes": ["string"],
                    "restriction": "allowlist",
                },
                "merchant_category_rule": {
                    "merchant_categories": ["string"],
                    "restriction": "allowlist",
                },
                "merchant_rule": {
                    "merchants": ["string"],
                    "restriction": "allowlist",
                },
                "spending_rule": {
                    "transaction_size_limit": {
                        "maximum": {"amount_cents": 0},
                        "minimum": {"amount_cents": 0},
                    },
                    "utilization_limit": {
                        "limit_amount": {"amount_cents": 0},
                        "preset": "daily",
                        "start_date": "startDate",
                        "timezone": "timezone",
                    },
                    "utilization_limit_v2": [
                        {
                            "limit_amount": {"amount_cents": 0},
                            "preset": "daily",
                            "start_date": "startDate",
                            "timezone": "timezone",
                        }
                    ],
                },
            },
            virtual_account_id="virtualAccountId",
        )
        assert_matches_type(CardGroup, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: SlashSDK) -> None:
        response = client.card_group.with_raw_response.create(
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        card_group = response.parse()
        assert_matches_type(CardGroup, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: SlashSDK) -> None:
        with client.card_group.with_streaming_response.create(
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            card_group = response.parse()
            assert_matches_type(CardGroup, card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: SlashSDK) -> None:
        card_group = client.card_group.retrieve(
            "cardGroupId",
        )
        assert_matches_type(CardGroup, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: SlashSDK) -> None:
        response = client.card_group.with_raw_response.retrieve(
            "cardGroupId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        card_group = response.parse()
        assert_matches_type(CardGroup, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: SlashSDK) -> None:
        with client.card_group.with_streaming_response.retrieve(
            "cardGroupId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            card_group = response.parse()
            assert_matches_type(CardGroup, card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `card_group_id` but received ''"):
            client.card_group.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_update(self, client: SlashSDK) -> None:
        card_group = client.card_group.update(
            card_group_id="cardGroupId",
        )
        assert_matches_type(CardGroup, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_update_with_all_params(self, client: SlashSDK) -> None:
        card_group = client.card_group.update(
            card_group_id="cardGroupId",
            name="name",
            spending_constraint={
                "country_rule": {
                    "countries": ["string"],
                    "restriction": "allowlist",
                },
                "merchant_category_code_rule": {
                    "merchant_category_codes": ["string"],
                    "restriction": "allowlist",
                },
                "merchant_category_rule": {
                    "merchant_categories": ["string"],
                    "restriction": "allowlist",
                },
                "merchant_rule": {
                    "merchants": ["string"],
                    "restriction": "allowlist",
                },
                "spending_rule": {
                    "transaction_size_limit": {
                        "maximum": {"amount_cents": 0},
                        "minimum": {"amount_cents": 0},
                    },
                    "utilization_limit": {
                        "limit_amount": {"amount_cents": 0},
                        "preset": "daily",
                        "start_date": "startDate",
                        "timezone": "timezone",
                    },
                    "utilization_limit_v2": [
                        {
                            "limit_amount": {"amount_cents": 0},
                            "preset": "daily",
                            "start_date": "startDate",
                            "timezone": "timezone",
                        }
                    ],
                },
            },
        )
        assert_matches_type(CardGroup, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_update(self, client: SlashSDK) -> None:
        response = client.card_group.with_raw_response.update(
            card_group_id="cardGroupId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        card_group = response.parse()
        assert_matches_type(CardGroup, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_update(self, client: SlashSDK) -> None:
        with client.card_group.with_streaming_response.update(
            card_group_id="cardGroupId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            card_group = response.parse()
            assert_matches_type(CardGroup, card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_update(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `card_group_id` but received ''"):
            client.card_group.with_raw_response.update(
                card_group_id="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: SlashSDK) -> None:
        card_group = client.card_group.list()
        assert_matches_type(CardGroupListResponse, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_list_with_all_params(self, client: SlashSDK) -> None:
        card_group = client.card_group.list(
            cursor="cursor",
            filter_name="filter:name",
        )
        assert_matches_type(CardGroupListResponse, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: SlashSDK) -> None:
        response = client.card_group.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        card_group = response.parse()
        assert_matches_type(CardGroupListResponse, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: SlashSDK) -> None:
        with client.card_group.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            card_group = response.parse()
            assert_matches_type(CardGroupListResponse, card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_method_get_utilization(self, client: SlashSDK) -> None:
        card_group = client.card_group.get_utilization(
            "cardGroupId",
        )
        assert_matches_type(CardGroupUtilization, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get_utilization(self, client: SlashSDK) -> None:
        response = client.card_group.with_raw_response.get_utilization(
            "cardGroupId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        card_group = response.parse()
        assert_matches_type(CardGroupUtilization, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get_utilization(self, client: SlashSDK) -> None:
        with client.card_group.with_streaming_response.get_utilization(
            "cardGroupId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            card_group = response.parse()
            assert_matches_type(CardGroupUtilization, card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_get_utilization(self, client: SlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `card_group_id` but received ''"):
            client.card_group.with_raw_response.get_utilization(
                "",
            )


class TestAsyncCardGroup:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncSlashSDK) -> None:
        card_group = await async_client.card_group.create(
            name="name",
        )
        assert_matches_type(CardGroup, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        card_group = await async_client.card_group.create(
            name="name",
            spending_constraint={
                "country_rule": {
                    "countries": ["string"],
                    "restriction": "allowlist",
                },
                "merchant_category_code_rule": {
                    "merchant_category_codes": ["string"],
                    "restriction": "allowlist",
                },
                "merchant_category_rule": {
                    "merchant_categories": ["string"],
                    "restriction": "allowlist",
                },
                "merchant_rule": {
                    "merchants": ["string"],
                    "restriction": "allowlist",
                },
                "spending_rule": {
                    "transaction_size_limit": {
                        "maximum": {"amount_cents": 0},
                        "minimum": {"amount_cents": 0},
                    },
                    "utilization_limit": {
                        "limit_amount": {"amount_cents": 0},
                        "preset": "daily",
                        "start_date": "startDate",
                        "timezone": "timezone",
                    },
                    "utilization_limit_v2": [
                        {
                            "limit_amount": {"amount_cents": 0},
                            "preset": "daily",
                            "start_date": "startDate",
                            "timezone": "timezone",
                        }
                    ],
                },
            },
            virtual_account_id="virtualAccountId",
        )
        assert_matches_type(CardGroup, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.card_group.with_raw_response.create(
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        card_group = await response.parse()
        assert_matches_type(CardGroup, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.card_group.with_streaming_response.create(
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            card_group = await response.parse()
            assert_matches_type(CardGroup, card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSlashSDK) -> None:
        card_group = await async_client.card_group.retrieve(
            "cardGroupId",
        )
        assert_matches_type(CardGroup, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.card_group.with_raw_response.retrieve(
            "cardGroupId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        card_group = await response.parse()
        assert_matches_type(CardGroup, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.card_group.with_streaming_response.retrieve(
            "cardGroupId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            card_group = await response.parse()
            assert_matches_type(CardGroup, card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `card_group_id` but received ''"):
            await async_client.card_group.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_update(self, async_client: AsyncSlashSDK) -> None:
        card_group = await async_client.card_group.update(
            card_group_id="cardGroupId",
        )
        assert_matches_type(CardGroup, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        card_group = await async_client.card_group.update(
            card_group_id="cardGroupId",
            name="name",
            spending_constraint={
                "country_rule": {
                    "countries": ["string"],
                    "restriction": "allowlist",
                },
                "merchant_category_code_rule": {
                    "merchant_category_codes": ["string"],
                    "restriction": "allowlist",
                },
                "merchant_category_rule": {
                    "merchant_categories": ["string"],
                    "restriction": "allowlist",
                },
                "merchant_rule": {
                    "merchants": ["string"],
                    "restriction": "allowlist",
                },
                "spending_rule": {
                    "transaction_size_limit": {
                        "maximum": {"amount_cents": 0},
                        "minimum": {"amount_cents": 0},
                    },
                    "utilization_limit": {
                        "limit_amount": {"amount_cents": 0},
                        "preset": "daily",
                        "start_date": "startDate",
                        "timezone": "timezone",
                    },
                    "utilization_limit_v2": [
                        {
                            "limit_amount": {"amount_cents": 0},
                            "preset": "daily",
                            "start_date": "startDate",
                            "timezone": "timezone",
                        }
                    ],
                },
            },
        )
        assert_matches_type(CardGroup, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.card_group.with_raw_response.update(
            card_group_id="cardGroupId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        card_group = await response.parse()
        assert_matches_type(CardGroup, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.card_group.with_streaming_response.update(
            card_group_id="cardGroupId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            card_group = await response.parse()
            assert_matches_type(CardGroup, card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_update(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `card_group_id` but received ''"):
            await async_client.card_group.with_raw_response.update(
                card_group_id="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncSlashSDK) -> None:
        card_group = await async_client.card_group.list()
        assert_matches_type(CardGroupListResponse, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSlashSDK) -> None:
        card_group = await async_client.card_group.list(
            cursor="cursor",
            filter_name="filter:name",
        )
        assert_matches_type(CardGroupListResponse, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.card_group.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        card_group = await response.parse()
        assert_matches_type(CardGroupListResponse, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.card_group.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            card_group = await response.parse()
            assert_matches_type(CardGroupListResponse, card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_utilization(self, async_client: AsyncSlashSDK) -> None:
        card_group = await async_client.card_group.get_utilization(
            "cardGroupId",
        )
        assert_matches_type(CardGroupUtilization, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get_utilization(self, async_client: AsyncSlashSDK) -> None:
        response = await async_client.card_group.with_raw_response.get_utilization(
            "cardGroupId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        card_group = await response.parse()
        assert_matches_type(CardGroupUtilization, card_group, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get_utilization(self, async_client: AsyncSlashSDK) -> None:
        async with async_client.card_group.with_streaming_response.get_utilization(
            "cardGroupId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            card_group = await response.parse()
            assert_matches_type(CardGroupUtilization, card_group, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_get_utilization(self, async_client: AsyncSlashSDK) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `card_group_id` but received ''"):
            await async_client.card_group.with_raw_response.get_utilization(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .webhook import Webhook
from .._models import BaseModel
from .pagination_response import PaginationResponse

__all__ = ["WebhookListResponse"]


class WebhookListResponse(BaseModel):
    items: List[Webhook]

    metadata: Optional[PaginationResponse] = None
    """Response sent when requesting a list of data"""
